#
# raccp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Library General Public License.
#

require 'racc/parser'

module Racc

  class RaccParser < Parser

    def initialize( racc )
      @racc = racc
      @interf    = racc.interf
      @ruletable = racc.ruletable
    end


    attr :start
    attr :classname

    attr :filename

    def parse( str, fname, mydebug = false )
      fname.must String
      
      @scanner = RaccScanner.new( str )
      @scanner.debug    = @yydebug  = mydebug
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      case val
      when Fixnum then v = val.id2name
      when String then v = val
      else
        v = val.inspect
      end
      raise ParseError,
%=parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'=
    end

##### racc 0.11.0 generates ###

Racc_reduce_table = [
0, 0, :racc_error,
6, 20, :_reduce_1,
0, 21, :_reduce_none,
2, 21, :_reduce_none,
3, 23, :_reduce_4,
1, 23, :_reduce_none,
2, 23, :_reduce_6,
2, 24, :_reduce_7,
3, 24, :_reduce_8,
3, 25, :_reduce_9,
3, 25, :_reduce_10,
1, 27, :_reduce_none,
2, 27, :_reduce_none,
2, 28, :_reduce_13,
2, 28, :_reduce_14,
2, 28, :_reduce_15,
1, 29, :_reduce_16,
2, 29, :_reduce_17,
1, 26, :_reduce_18,
1, 26, :_reduce_19,
2, 22, :_reduce_none,
3, 22, :_reduce_none,
3, 30, :_reduce_22,
3, 30, :_reduce_23,
0, 31, :_reduce_24,
2, 31, :_reduce_25,
1, 32, :_reduce_26,
1, 32, :_reduce_27,
1, 32, :_reduce_28,
2, 32, :_reduce_29 ]

Racc_reduce_n = 30

Racc_shift_n = 52

Racc_action_table = [
     2,     2,    -1,   -30,     0,     3,    -1,   -30,     3,     4,
    -1,   -30,     0,     5,    -1,   -30,    -1,    -2,    -1,    52,
    10,     7,     6,     9,     4,    11,     7,    12,     9,    13,
    -1,   -30,    12,    14,    11,    17,    13,    18,    -1,   -30,
    -1,    -3,     3,    20,     8,    19,    -1,   -30,    -1,    -5,
     3,    23,    -1,   -30,     3,    26,    -1,   -30,    12,    14,
    11,    17,    13,    18,    -1,   -30,     3,    20,     8,    19,
    -1,   -30,    12,    14,    11,    17,     9,    31,    13,    18,
    -1,   -30,    -1,   -11,     3,    20,     8,    19,    -1,   -30,
     3,    20,     8,    19,    -1,   -30,    -1,   -19,    -1,   -18,
     3,    20,     8,    19,     5,    35,    -1,   -30,     8,    36,
    -1,   -30,    15,    37,    -1,   -30,    16,    38,    14,    39,
    -1,   -30,     3,    23,     5,    41,    -1,   -30,    -1,    -6,
    12,    14,    10,    42,    11,    17,    13,    18,    -1,   -30,
     3,    20,     8,    19,    -1,   -14,    -1,   -16,    -1,   -12,
    -1,   -10,     3,    20,     8,    19,    -1,   -13,     3,    20,
     8,    19,    -1,   -15,     8,    44,    -1,   -30,    -1,    -4,
    -1,    -7,    -1,   -24,    -1,   -24,    -1,   -20,    16,    38,
    14,    47,    -1,   -30,    -1,    -1,    -1,    -9,    -1,   -17,
    -1,    -8,     3,    50,     8,    49,    17,    48,    18,    51,
    -1,   -22,     3,    50,     8,    49,    17,    48,    18,    51,
    -1,   -23,    -1,   -21,    -1,   -28,    -1,   -27,    -1,   -26,
     3,    20,     8,    19,    -1,   -30,    -1,   -25,    -1,   -29 ]

Racc_action_table_ptr = [
     0,     4,     8,    12,    16,    18,    20,    32,    40,    42,
    48,    50,    54,    58,    66,    72,    82,    84,    90,    96,
    98,   100,   108,   112,   116,   122,   128,   130,   140,   146,
   148,   150,   152,   158,   164,   168,   170,   172,   174,   176,
   178,   184,   186,   188,   190,   192,   202,   212,   214,   216,
   218,   220,   226,   228 ]


Racc_goto_table = [
    20,     1,    21,     6,    23,     8,    25,    10,    27,    15,
    28,    16,    24,    21,    26,    22,    30,    24,    22,    25,
    27,    27,    28,    16,    29,    28,    26,    29,    28,    30,
    29,    32,    26,    29,    29,    33,    26,    29,    26,    34,
    30,    40,    28,    30,    26,    43,    26,    43,    26,    43,
    31,    45,    31,    46,    32,    52,    32,    52,    26,    53,
    -1,    -1 ]

Racc_goto_table_ptr = [
     0,    -1,    -1,    -1,     2,    -1,     4,     8,    -1,    12,
    -1,    16,    -1,    20,    24,    28,    -1,    30,    34,    -1,
    -1,    38,    -1,    -1,    -1,    40,    -1,    42,    44,    -1,
    -1,    -1,    46,    48,    -1,    -1,    -1,    50,    52,    -1,
    -1,    -1,    -1,    -1,    -1,    54,    56,    -1,    -1,    -1,
    -1,    58,    -1,    -1 ]


Racc_token_table = {
 false => 0,
 Object.new => 1,
 :CLASS => 2,
 :TOKEN => 3,
 :RULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :START => 7,
 :STRING => 8,
 :PRECHIGH => 9,
 :PRECLOW => 10,
 :LEFT => 11,
 :RIGHT => 12,
 :NONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 :ACTION => 17,
 "=" => 18 }

Racc_debug_parser = false

##### racc system variables end #####

 # reduce 0 omitted

 module_eval( '', '(build)', 79 ) {
  def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = (Integer === val[1]) ? val[1].id2name : val[1]
   result
  end
 }

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( '', '(build)', 90 ) {
  def _reduce_4( val, _values, result )
      @interf.end_register_conv
   result
  end
 }

 # reduce 5 omitted

 module_eval( '', '(build)', 96 ) {
  def _reduce_6( val, _values, result )
      if @start then
        raise ParseError, "start rule defined twice"
      end
      @ruletable.start = @start = @interf.get_token( val[1] )
   result
  end
 }

 module_eval( '', '(build)', 104 ) {
  def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
   result
  end
 }

 module_eval( '', '(build)', 108 ) {
  def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
   result
  end
 }

 module_eval( '', '(build)', 113 ) {
  def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
   result
  end
 }

 module_eval( '', '(build)', 117 ) {
  def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
   result
  end
 }

 # reduce 11 omitted

 # reduce 12 omitted

 module_eval( '', '(build)', 127 ) {
  def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 131 ) {
  def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 135 ) {
  def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
   result
  end
 }

 module_eval( '', '(build)', 140 ) {
  def _reduce_16( val, _values, result )
      result = val
   result
  end
 }

 module_eval( '', '(build)', 144 ) {
  def _reduce_17( val, _values, result )
      result.push val[1]
   result
  end
 }

 module_eval( '', '(build)', 149 ) {
  def _reduce_18( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 153 ) {
  def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
   result
  end
 }

 # reduce 20 omitted

 # reduce 21 omitted

 module_eval( '', '(build)', 163 ) {
  def _reduce_22( val, _values, result )
      @interf.register_rule @interf.get_token(val[0]), val[2]
   result
  end
 }

 module_eval( '', '(build)', 167 ) {
  def _reduce_23( val, _values, result )
      @interf.register_rule nil, val[2]
   result
  end
 }

 module_eval( '', '(build)', 172 ) {
  def _reduce_24( val, _values, result )
      result = []
   result
  end
 }

 module_eval( '', '(build)', 176 ) {
  def _reduce_25( val, _values, result )
      result.push val[1] if val[1]
   result
  end
 }

 module_eval( '', '(build)', 181 ) {
  def _reduce_26( val, _values, result )
      result = @interf.get_token( val[0] )
   result
  end
 }

 module_eval( '', '(build)', 185 ) {
  def _reduce_27( val, _values, result )
      result = @interf.get_token( eval %<"#{result}"> )
   result
  end
 }

 module_eval( '', '(build)', 189 ) {
  def _reduce_28( val, _values, result )
      result = Action.new( *result )
   result
  end
 }

 module_eval( '', '(build)', 193 ) {
  def _reduce_29( val, _values, result )
      @interf.register_tmpprec val[1]
      result = nil
   result
  end
 }

  end   # class RaccParser

end   # module Racc
